import random
import pprint
import sys

# Each insn is a python dictionary (map) with the following fields:
# 
#   src1 - operand #1 source register id
#   src2 - operand #2 source register id
#   dst - destination register id
# 
# Register id's are just integers.  There are 16 architectural
# registers in this simple ISA, with id's 0 to 15.
# 
# There are two additional fields which you should fill in as you
# schedule instructions according to their dependences.
#
#   consumer_insns - list of insns that consume output of this insn
#   depth - depth of insn in the scheduling tree

def gen_insns(n):
    # Unique root insn to make sure our dependencies form a tree.
    insns = [ {'src1':0, 'src2':0, 'dst':0,
               'consumer_insns':[], 'depth':0} ]

    live_regs = set([0]) # start out with only 1 live reg
    for i in range(n):
        insn = {}
        insn['src1'] = random.choice(list(live_regs))
        insn['src2'] = random.choice(list(live_regs))
        insn['dst'] = random.randint(0, 15)
        live_regs.add(insn['dst'])

        # Used for building dependence tree.
        insn['consumer_insns'] = []

        # Used for calculating program latency.
        insn['depth'] = None

        insns.append(insn)
    return insns

def rename(insn):
    # TODO: your code here
    pass

def compute_latency(insns):
    # TODO: your code here
    pass

def compute_max_width(insns):
    # TODO: your code here
    pass

def compute_max_pregs(insns):
    # TODO: your code here
    pass

def main(insns):
    # First, rename insns to remove false dependences.
    for i in insns:
        rename(i)
     
    results = {}

    # Compute latency (versus serial).
    results['latency'] = compute_latency(insns)
    results['serial latency'] = len(insns)

    # Compute max machine width used (max number of insns that
    # executed in parallel).
    results['max machine width used'] = compute_max_width(insns)

    # Compute max number of pregs used.
    results['max pregs used'] = compute_max_pregs(insns)

    return repr(results)

if __name__ == "__main__":
    # Edit this line to run with different trace files (or pass a
    # filename on the command name).
    filename = "serial-6ticks-1width-3pregs.insns"
    if len(sys.argv) > 1:
        filename = sys.argv[1]
    pprint.pprint(main( eval(open(filename).read()) ))

    # Uncomment this line to run with a random trace instead.
    # pprint.pprint(main( gen_insns(5) ))

    # This code below will dump a random trace of 5 insns to the
    # terminal, so you can save it as a file and read it back in later.
    # pprint.pprint(gen_insns(5))
